function eis_cal::warm_pixels,data,iwin,init=init
  if n_elements(init) eq 1 then return,-1
; 
; first find the last wp dir that preceeds the observation
;
  dirsep = get_delim()
  date=anytim2tai((data->gethdr())->getdate_obs())
  path=getenv('EIS_CAL_DATA')+dirsep+'wp'+dirsep 
  pathlen=strlen(path)
  wp_dirs=file_search(path+'*',/test_directory) 
  if n_elements(wp_dirs) eq 0 then begin
    if iwin eq 0 then message,'no warm pixel directory found ',/info
    return,-1
  endif
  wp_dates=anytim2tai(strmid(wp_dirs,pathlen,10))
;
; dirindex = max(where (wp_dates le date)) > 0
  dum=min(wp_dates-date,dirindex,/abs)
;
  self.wp_dir=wp_dirs[dirindex]+dirsep
  if iwin eq 0 then begin
    message,'Warm pixels dir: '+self.wp_dir,/info
  endif
;
; find right quadrant and choose the file prefix accordingly
  data->getwin,iwin,wd,pos
  quad=self->quadrant(pos)
  case quad of 
  0:pref='coords_b_left'
  1:pref='coords_b_right'
  2:pref='coords_a_left'
  3:pref='coords_a_right'
  else:
  endcase
; find the max exposure time and choose the file suffix accordingly
  exp_dur=data->getexp()
  exp_dur=max(data->getexp())
  suff=[30,60,100]
  isuff=min(where (suff gt exp_dur-1))
  min_exp_dur=min(data->getexp())
  min_isuff=min(where (suff gt min_exp_dur-1))
  if min_isuff lt isuff and iwin eq 0 then begin
    message,"WARNING  Exposures have different duration.",/info
    message,"Warm pixels taken according to the longest.",/info
  endif
  if isuff eq -1 then begin
    if iwin eq 0 then begin
      message,"Some exposures last longer than warm pixel's ones.",/info
      message,"The longest warm pixels exposure has been selected.",/info
    endif
    isuff = n_elements(suff)-1
  endif
; check if the wp files are the first ones which cover the CCDs' middle strip
  cm = 0
  wp_dir=wp_dirs[dirindex]+dirsep
  fm=file_search(wp_dir+pref+'*middle*',count=cm)
  if cm eq 1 then begin
   left  = [50L,1074L,2198L,3222L]
   restore,fm[0] ; restores ccd_data - UINT  = Array[1024, 512]
   a = pos[0]-left[quad]
   b = pos[0]+pos[1]-left[quad]-1
   c = pos[2]-256L
   d = pos[2]+pos[3]-257L
   ccd_data_info = size(ccd_data)
   mc = 0L
   md = ccd_data_info[2]-1L
   if c lt mc then begin
     message,"WARNING Window extending below of the warm pixels matrix",/info
     message,"        Part exceeding not treated",/info
     c=mc
   endif
   if d gt md then begin
     message,"WARNING Window extending above of the warm pixels matrix",/info
     message,"        Part exceeding not treated",/info
     d=md
   endif
   wp=ccd_data[a:b,c:d]
   return, wp
  endif
; alternatively, now find the CCD's bottom and top files
  cb = 0
  index = dirindex
  repeat begin
    wp_dir=wp_dirs[index]+dirsep
    is = isuff
    repeat begin
    fb=file_search(wp_dir+pref+'*bottom*'+'_'+strtrim(suff[is],2)+'s*',count=cb)
    is = is + 1
    endrep until cb gt 0 or is eq n_elements(suff)
    index = index + 1
  endrep until cb gt 0 or index eq n_elements(wp_dirs)
  if cb eq 0 and dirindex gt 0 then begin
   index = dirindex-1
   repeat begin
     wp_dir=wp_dirs[index]+dirsep
     is = isuff
     repeat begin
     fb=file_search(wp_dir+pref+'*bottom*'+'_'+strtrim(suff[is],2)+'s*',count=cb)
     is = is + 1
     endrep until cb gt 0 or is eq n_elements(suff)
     index = index - 1
   endrep until cb gt 0 or index lt 0
  endif
  if cb eq 1 and iwin eq 0 and wp_dir ne self.wp_dir then $
    message,'Warm pixel CCD bottom files from '+wp_dir,/info
;
  ct = 0
  index = dirindex
  repeat begin
    wp_dir=wp_dirs[index]+dirsep
    is = isuff
    repeat begin
    ft=file_search(wp_dir+pref+'*top*'+'_'+strtrim(suff[is],2)+'s*',count=ct)
    is = is + 1
    endrep until ct gt 0 or is eq n_elements(suff)
    index = index + 1
  endrep until ct gt 0 or index eq n_elements(wp_dirs)
  if ct eq 0 and dirindex gt 0 then begin
   index = dirindex-1
   repeat begin
     wp_dir=wp_dirs[index]+dirsep
     is = isuff
     repeat begin
     ft=file_search(wp_dir+pref+'*top*'+'_'+strtrim(suff[is],2)+'s*',count=ct)
     is = is + 1
     endrep until ct gt 0 or is eq n_elements(suff)
     index = index - 1
   endrep until ct gt 0 or index lt 0
  endif
  if ct eq 1 and iwin eq 0 and wp_dir ne self.wp_dir then $ 
     message,'Warm pixel CCD top files from '+wp_dir,/info
  if cb eq 0 and ct eq 0 then begin
   if iwin eq 0 then message,'No warm pixel files found',/info
   return,-1
  endif
; now construct the warm pixel matrix
  left  = [50L,1074L,2198L,3222L]
  wp=uintarr(pos[1],pos[3])
  a = pos[0]-left[quad]
  b = pos[0]+pos[1]-left[quad]-1L
  if cb eq 1 then begin
   restore,fb[0]
   c = pos[2]
   d = pos[2]+pos[3]-1L
   ccd_data_info = size(ccd_data)
   md = ccd_data_info[2]-1L
   if c le md then begin
     if d gt md then d=md
;There is an offset in the bottom maps: they cover [1:512] instead of [0:511]
     wp[0:b-a,0:d-c]=ccd_data[a:b,c-1:d-1]
   end
  endif else if iwin eq 0 then $
      message,"WARNING Bottom half of the warm pixels matrix missing",/info
  if ct eq 1 then begin
   restore,ft[0]
   c = pos[2]-512L
   d = pos[2]+pos[3]-513L
   if d ge 0 then begin
     if c lt 0 then c=0
     wp[0:b-a,pos[3]-1-(d-c):pos[3]-1]=ccd_data[a:b,c:d]
   end
  endif else if iwin eq 0 then $
      message,"WARNING Top half of the warm pixels matrix missing",/info
  return, wp
end
