;+
; NAME:
;       EIS_HK__DEFINE
;
; PURPOSE:
;       The eis_hk object is the general superclass for Solar-B/EIS 
;       House Keeping data (hk)
;       class objects used by the Hansteen/Wikstl QL/data analysis
;       SW. The hk object contains data that is aqcuired from 
;       the House Keeping telemetry stream on Solar-B. 
;       Some of these are used in the main FITS hdr of Solar-B/EIS fits files.
;
; CATEGORY:
;       Hansteen/Wikstol Data analysis SW
;
; CALLING SEQUENCE:
;	hk_obj = obj_new('hk', file = file)
;
; INPUTS:
;      None
;
; KEYWORD PARAMETERS:
;
;
; OUTPUTS:
;       Objects of class EIS_HK includes the following parameters:
;
; CALLS:
;
; COMMON BLOCKS:
;
;
; PROCEDURE:
;       Defines the structure of plan class objects. If no filename 
;       is given as input, the FITS keywords are set to defaults by 
;       the init-procedure. If filename is given (house keeping 
;       telemetry file name), it is read from that file by eis_hk__read.pro
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;       16-Dec-2005: Oivind Wikstol - Empty object created.
;        4-Jul-2008: A. Gardini     - Added HELP and INIT_EIS_HK_METHODS.
;-
function eis_hk::init, file = file
  self->init_eis_hk_methods
  if n_elements(file) eq 0 then begin
    self.tr_mode = 'NA'    
  endif else begin
    self-> read, file
  endelse
  return,1
end

pro eis_hk::read, file, init=init
  if n_elements(init) eq 1 then return
  return
end

function eis_hk::get_tr_mode, init=init
  if n_elements(init) eq 1 then return, -1
  return, self.tr_mode
end

pro eis_hk::help
  help,self,/obj
  return
end

pro eis_hk::init_eis_hk_methods

  self->read,/init
  a=self->get_tr_mode(/init)

  return
end

pro eis_hk__define
  struct={eis_hk, $
          tr_mode:' '}
end
