;+
; NAME:
;       EIS_PLAN__DEFINE
;
; PURPOSE:
;       The eis_plan object is the general superclass for plan
;       class objects used by the Hansteen/Wikstl QL/data analysis
;       SW. The plan object contains data that is aqcuired from 
;       an instruments planning tool. These data should be found 
;       in the main header unit (HDU) of the fits-files 
;
;       For a specific instrument, a new class must be defined
;       (i.e. eis_plan__define.pro for SOLAR-B EIS instrument). This class
;       inherits the eis_plan superclass. The plan class must
;       provide a read-routine for reading the specific plan
;       (i.e. eis_plan__read.pro).
;
;
; CATEGORY:
;       Hansteen/Wikstol Data analysis SW
;
; CALLING SEQUENCE:
;
;
; INPUTS:
;      None
;
; KEYWORD PARAMETERS:
;
;
; OUTPUTS:
;       Objects of class EIS_PLAN includes the following parameters:
;
; CALLS:
;
;      eis_get_sequence_details
;      eis_get_study_details
;      eis_get_line_list_details
;      eis_get_raster_details
;
; COMMON BLOCKS:
;
;
; PROCEDURE:
;       Defines the structure of plan class objects. If no filename 
;       is given as input, the FITS keywords are set by 
;       the init-procedure. If filename is given (plan or as-run
;       file), it is read from that file by eis_plan__read.pro
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;       28-Nov-2002: Oivind Wikstol  - First draft
;       13-Jan-2003: Oivind Wikstol  - Modifications to the init and
;                                      read methods. Add line_id to
;                                      the data structure
;       12-Apr-2005: Oivind Wikstol  - Added read method. 
;                                      Changed object definitions and almost 
;                                      completely re-written because of access
;                                      to planning tool data. 
;       August 2006: Oivind Wikstol  - Almost completely re-written to work with 
;                                      new planning tool
;       20-Nov-2006: Viggo H/        - Small clean up and last dry run test
;                    John Rainnie      at ISAS.                                      
;       23-Nov-2006: Viggo Hansteen  - Added method for reading/finding whether there 
;                                      is SAA or HLZ during the observation
;       10-Feb-2007: Viggo Hasnteen  - Added help methods.
;       15-Mar-2007: Viggo Hansteen/ - Bug fix in getsaa
;                    Matthew Whillock/
;                    John Rainnie    
;        4-Jul-2008: A. Gardini      - Added HELP and INIT_EIS_PLAN_METHODS.
;
; $Id: eis_plan__define.pro 4173 2014-09-17 15:31:24Z viggoh $
;-

pro eis_plan__define
  nwin = 25
  struct={eis_plan, $
          help:obj_new(), $
          telescop:' ', $
          instrume:' ', $
          rast_acr:' ', $
          study_acr:' ', $
          study_auth:' ', $
          obstitle:' ', $
          obs_dec: ' ', $
          solb_sci:' ', $
          eis_sci:' ', $
          observer:' ', $
          planner:' ', $
          tohban:' ', $
          target:' ', $
          sci_obj:' ', $
          join_sb:' ', $
          obs_id:0L, $
          jop_id:0, $
          noaa_num:-1L, $
          study_id:0L, $
          datatype:' ', $
          ll_acr:' ', $          
          line_id:strarr(nwin), $
          line_wvl:fltarr(nwin), $
          exptime:0.0, $
          ll_id:0L, $
          rast_id:0L, $
          saa:' ', $
          hlz:' ', $
          tl_id:0L, $
          tl_id_hunter:0L}
end
