pro eis_plan::read, seq_id, rast_id,init=init
  if n_elements(init) eq 1 then return
  if n_params() lt 2 then begin
    !quiet=0
    message,'SEQ_ID or RAST_ID missing. Using default planning values.',/info
    !quiet=1
    self-> default
    return
  endif

  ; 20130220 mcrw
;  message, 'read SEQ_ID is '+strtrim(string(seq_id),2)+', RAST_ID is '+strtrim(string(rast_id),2),/info

; make sure seq_id is long:
  seq_id = long(seq_id)

  zdbase = fix_zdbase(/USER) 

; get sequence structure based on seq_id
  if seq_id lt 5 then begin
;;; mcrw 20130219 - 20130209 plan crash. Is seq_id out of range?
     self->debug_msg, 'HUNTER, ', seq_id, rast_id
;    !quiet=0
;    message, 'HUNTER, SEQ_ID is '+strtrim(string(seq_id),2)+', RAST_ID is '+strtrim(string(rast_id),2),/info
;    !quiet=1
    self.tl_id_hunter = self.tl_id
    seq = eis_get_science_entry(seq_id) ; should add response keyword
  endif else begin
;;; mcrw 20130219 - 20130209 plan crash. Is seq_id out of range?
     self->debug_msg, 'NON-HUNTER1, ', seq_id, rast_id
;    !quiet=0
;    message, 'NON-HUNTER1, SEQ_ID is '+strtrim(string(seq_id),2)+', RAST_ID is '+strtrim(string(rast_id),2),/info
;    !quiet=1
    seq = eis_get_science_entry(seq_id)
;;; mcrw 20130219 - 20130209 plan crash. Is seq_id out of range?
    self->debug_msg, 'NON-HUNTER2, ', seq_id, rast_id
;    !quiet=0
;    message, 'NON-HUNTER2, SEQ_ID is '+strtrim(string(seq_id),2)+', RAST_ID is '+strtrim(string(rast_id),2),/info
;    !quiet=1
  endelse

  if n_tags(seq) eq 0 then begin
    !quiet=0
    message,'Could not find sequence '+string(seq_id)+' in database.',/info
    message,'Using default planning values',/info
    !quiet=1
;;; mcrw 20130219 - 20130209 plan crash. Is seq_id out of range?
    self->debug_msg, 'NO PLAN, ', seq_id, rast_id
;    message, 'NO PLAN, SEQ_ID is '+strtrim(string(seq_id),2)+', RAST_ID is '+strtrim(string(rast_id),2),/info
;    !quiet=1
    self-> default
    return
  endif

;;; mcrw 20140808 - if the planner does not make the target to be
;;;                 'Engineering' then the reformatter will crash 
;;;                 trying to fill the raster details
  if (strtrim(seq.target,2) EQ 'Engineering') THEN BEGIN
;;; mcrw 20130219 - 20130209 plan crash. Is seq_id out of range?
     self->debug_msg, 'Target is ENG, ', seq_id, rast_id
;    !quiet=0
;    message, 'Target ENG, SEQ_ID is '+strtrim(string(seq_id),2)+', RAST_ID is '+strtrim(string(rast_id),2),/info
;    !quiet=1
    rast_id = 0
  ;;; mcrw 20130219 - 20130209 plan crash. Is seq_id out of range?
    self->debug_msg, 'ENG, ', seq_id, rast_id
;    !quiet=0
;    message, 'ENG, SEQ_ID is '+strtrim(string(seq_id),2)+', RAST_ID is '+strtrim(string(rast_id),2),/info
;    !quiet=1
    self->default
  endif
  
;;; mcrw 20140808 - workaround to detect engineering rasters in case
;;;                 target has been changed in the plan
  change_raster_id = 0
  case rast_id of
     49271L : change_raster_id = 1 ; C077
     49266L : change_raster_id = 1 ; C072
     49265L : change_raster_id = 1 ; C071
     else :
  endcase
  if change_raster_id eq 1 then begin
     self->debug_msg, 'ENG (BAD TARGET), ', seq_id, rast_id
;     !quiet=0
;     message, 'ENG (BAD TARGET), SEQ_ID is '+strtrim(string(seq_id),2)+', RAST_ID is '+strtrim(string(rast_id),2),/info
;     !quiet=1
     rast_id = 0
  end

;;; mcrw 20130219 - 20130209 plan crash. Is seq_id out of range?
  self->debug_msg, 'Updating plan values: ', seq_id, rast_id
;  !quiet=0
;  message, 'Updating plan values: SEQ_ID is '+strtrim(string(seq_id),2)+', RAST_ID is '+strtrim(string(rast_id),2),/info
;  !quiet=1

  self.tl_id = seq_id
  self.observer = seq.observer
  self.planner = seq.planner 
  self.tohban = seq.tohbans[0]
  self.noaa_num = seq.target_id 
  self.sci_obj = seq.sci_obj
  self.obs_dec = seq.obs_dec
;  self.eis_sci = seq.eis_science
  self.solb_sci = seq.solarb_science
  self.obstitle = seq.obstitle
  self.join_sb = seq.join_sb
  self.obs_id = seq.obs_id
  self.jop_id = seq.jop_id
  zdbase = fix_zdbase(/EIS)

  ; get study ID and study details based on seq_id
  study_id = seq.study_id
  st = db_read_study_entry(study_id)
  self.study_id = study_id
  self.study_acr = st.acronym
  self.study_auth = st.author
  self.target = st.target
  self.datatype = st.category
  
  if (rast_id NE 0) then begin
;;; mcrw 20140806 - 20140726 plan crash. Is seq_id out of range?
;;; Also self.rast_id is not set if rast_id eq 0...
     self->debug_msg, 'Rast id ne 0, updating raster values: ', seq_id, rast_id
;     !quiet=0
;     message, 'Rast id ne 0, updating raster values', /info
;     !quiet=1
    ; get raster structure from rast_id:
      rast = db_read_raster_entry(rast_id)
      self.rast_id = rast_id
      self.rast_acr = rast.acronym
  ; of up to 8 exposures times, pick the first
      exposures = rast.exposures
      self.exptime = exposures[0]
      self.ll_id = rast.ll_id
      nlines = rast.nwindows

; get line list info:
;;; mcrw 20140806 - 20140726 plan crash. Is seq_id out of range?
      self->debug_msg, 'Getting linelist(s) for raster: ', seq_id, rast_id
;      !quiet=0
;      message, 'Getting linelist(s) for raster', /info
;      !quiet=1
      ll = db_read_linelist_entry(self.ll_id)
      self.ll_acr = ll.acronym
      for i = 0,nlines -1 do begin
          linfo = ll.lines[i]
          self.line_id[i] = strjoin([linfo.name, $
                                     strcompress(linfo.wavelength/100., /remove_all)], ' ', /single) 
          self.line_wvl = linfo.wavelength/100.
      endfor
  endif
end
