pro eis_spcd::construct_xypos,aocs=aocs ,init=init
  if n_elements(init) eq 1 then return
if n_elements(aocs) eq 0 then aocs=0
self->setxyoffset 
date_obs=self->getdate_obs()
date_end=self->getdate_end()   
if aocs eq 0 then begin
;;;if aocs eq 1 then begin
  self->construct,'TIME_AOCS1'
  if self.ioerror then return
  self.time_pt=ptr_new(ulong(*self.time_aocs1))
  time_sub=where(*self.time_pt ge ulong(date_obs)-32ul and *self.time_pt le ulong(date_end)+32ul)
  if time_sub[0] eq -1 then begin
    time_sub=0
    message,'cannot find HK data around time of date_obs',/info
    message,'using parameters at time '+string((*self.time_pt)[time_sub])+', date_obs is '+string(date_obs),/info
  endif
  *self.time_pt=(*self.time_pt)[time_sub]
  scale=0.10986663
  origin=1800
  self->construct,'UFSAUSE'
  self->construct,'UFSBUSE'
; find which ultra fine sensor, a or b, is in use
  comp_xypos=0
  if self.ufsause then begin
    self->construct,'UFSAXANG'
    self->construct,'UFSAYANG'
    ufsxang=self->getufsaxang()
    ufsyang=self->getufsayang()
    if ufsxang[0] eq 0 or ufsyang[0] eq 0 then comp_xypos=0 else comp_xypos=1
  endif 
  if self.ufsbuse then begin
    self->construct,'UFSBXANG'
    self->construct,'UFSBYANG'
    ufsxang=self->getufsbxang()
    ufsyang=self->getufsbyang()
    if ufsxang[0] eq 0 or ufsyang[0] eq 0 then comp_xypos=0 else comp_xypos=1
  endif
  if not comp_xypos then begin
    message,'no ufss in use, no positioning computed',/info
    self.ioerror=1
    return
  endif
  xufss=ufsxang*scale-origin
  yufss=ufsyang*scale-origin
  self->construct,'ERRANGZ'
  thetaz=self->geterrangz()*0.0772476*!pi/180./60./60. ; 648000/2^23 in arcsec converted 
                                                       ; to radians
  xoffut=self->getxoffut()
  yoffut=self->getyoffut()
  xoffsu=self->getxoffsu()
  yoffsu=self->getyoffsu()
;
; offset angle of the telescope according to "Erratum of the list of Solar-B "Hinode"
; Mission-Wide keywords" Section: Note: Telemetry of AOCS data 2006/11/05 Masumi Shimojo
;
; xcen,ycen (coordinate of the center of the image) 
; 2006/11/14 viggoh: am assuming this is the same as xhelio, yhelio defined by Shimojo
; to find the position of a given pixel on EIS use 
; x = xcen+(npixx-nrefx)*pixszx*cos(inst_rot+thetaz)+(npixy-nrefy)*pixszy*sin(inst_rot+thetaz)
; y = ycen-(npixx-nrefx)*pixszx*sin(inst_rot+thetaz)+(npixy-nrefy)*pixszy*cos(inst_rot+thetaz)
; note that the crot keyword contains inst_rot+sat_rot equivalent to inst_rot+thetaz
;
  xcen=xoffsu+(xufss-xoffut)*cos(thetaz)+(yufss-yoffut)*sin(thetaz)
  ycen=yoffsu-(xufss-xoffut)*sin(thetaz)+(yufss-yoffut)*cos(thetaz)
  self.xcen=ptr_new(xcen[time_sub])
  self.ycen=ptr_new(ycen[time_sub])
endif else begin
  self->construct,'TIME_HK2'
  if self.ioerror then return
  self.time_pt=ptr_new(ulong(*self.time_hk2))
  time_sub=where(*self.time_pt ge ulong(date_obs)-32ul and *self.time_pt le ulong(date_end)+32ul)
  if time_sub[0] eq -1 then begin
    time_sub=0
    message,'cannot find HK data around time of date_obs',/info
    message,'using parameters at time '+string((*self.time_pt)[time_sub])+', date_obs is '+string(date_obs),/info
  endif
  *self.time_pt=(*self.time_pt)[time_sub]
  self->construct,'SC_ATT_X'
  self->construct,'SC_ATT_Y'
  self->construct,'SC_ATT_Z'
;
; Rather, for reformatter pointing keywords, which
; tells the heliocenteric coordinate of the observing region in order
; of 10 arcsec accuracy, the AOCS onboard determined attitude values
; available in HK2 packets may be more suitable. These values are
; the AOCS determined satellte attitude by using 2 UFSS sensors and
; Gyros with including instrument offset relative to the SOT pointing
; direction.
; [HK2_ATT_X]
;    Representing N-S direction (but sign is opposite)
;    APID 428  Word 275  Length 24bit
;    Conversion coefficients (Xarcsec= C1*X +C0; X telemetry)
;     C1: 180/2^23*60*60  for the conversion from telemetry to arcsec
;      C0: 0
; [HK2_ATT_Y]
;   Representing E-W direction (but sign is opposite)
;   APID 428  Word 278  Length 24bit
;   Conversion coefficients (Xarcsec= C1*X +C0; X telemetry)
;     C1: 180/2^23*60*60  for the conversion from telemetry to arcsec
;     C0: 0
; [HK2_ATT_Z]
;   Representing rotation around Z (sign is not yet confirmed)
;   APID 428  Word 281  Length 24bit
;   Conversion coefficients (Xarcsec= C1*X +C0; X telemetry)
;     C1: 180/2^23*60*60  for the conversion from telemetry to arcsec
;     C0: 0
; Shimizu-san e-mail of 04-Dec-2006...
;
  if not self.ioerror then begin
; First convert 24 bit to 32 bit long, change sign, and convert to arcsec
    xcen=-1.0*ishft(*self.att_y,8)/256.*180.*60.*60./2ul^23
    ycen=-1.0*ishft(*self.att_x,8)/256.*180.*60.*60./2ul^23
; Then add EIS specific offset
    xcen=xcen[time_sub]+self->getxoffut()
    ycen=ycen[time_sub]+self->getyoffut()
    self.xcen=ptr_new(xcen)
    self.ycen=ptr_new(ycen)
  endif
endelse
;
; crval (coordinate of the reference pixel in heliocentric coordinate)
; crpix (pixel coordinate of the reference pixel)
; 20061114 viggoh: don't know if crpix is defined, setting 
;                    crval eqal to xcen,ycen
; 20100217 viggoh: set crval1,crval2 to first presumed "good" pair of 
;                  xcen,ycen
;
limxy=self->getpointing_limits()
indx=where(xcen gt limxy.xmin and xcen lt limxy.xmax and $
           ycen gt limxy.ymin and ycen lt limxy.ymax,count)
if count eq 0 then begin
  message,'no good pointing found!',/info
endif else begin
  *self.crval1=xcen[min(indx)]
  *self.crval2=ycen[min(indx)]
endelse
return
end
