;+
; NAME:
;       EIS_SPCD__DEFINE
;
; PURPOSE:
;       The eis_spcd object is the general superclass for spcd
;       class objects used by the Hansteen/Wikstl QL/data analysis
;       SW. The spcd object contains data that is aqcuired from 
;       the space craft data telemetry stream on Solar-B. 
;       Some of these are used in the main FITS hdr of Solar-B fits files.
;
; CATEGORY:
;       Hansteen/Wikstol Data analysis SW
;
; CALLING SEQUENCE:
;	spcd_obj = obj_new('spcd', file = file)
;
; INPUTS:
;      None
;
; KEYWORD PARAMETERS:
;
;
; OUTPUTS:
;       Objects of class EIS_SPCD includes the following parameters:
;
; CALLS:
;
; COMMON BLOCKS:
;
;
; PROCEDURE:
;       Defines the structure of plan class objects. If no filename 
;       is given as input, the FITS keywords are set by 
;       the init-procedure. If filename is given (space craft data telemetry 
;       file), it is read from that file by eis_spcd__read.pro
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;       15-Nov-2005: Oivind Wikstol. First draft
;                    WARNING: saa, hlz, should come from ISACS-PLN files??
;       August 2006: Added xcen and ycen
;       14-Nov-2006: Viggo Hansteen - Added all variables needed for s/c pointing info
;                    WARNING: Still a bit unclear about difference between xcen,ycen
;                             and xhelio,yhelio
;                    saa, hlz will be moved to planning object
;       22-Nov-2006: Viggo Hansteen - Fixed pointing: xcen, ycen should now represent 
;                                     the solar x and y at the center of the image
;                                     Call to calibration object to get EIS offset
;                                     relative AOCS/SOT
;       23-Nov-2006: Viggo Hansteen - Moved saa and hlz to the planning object.
;       03-Dec-2006: Viggo Hansteen - Improved error handling in case "eis_aocs1" file is not
;                                     found. Also constrain search in "tomorrow's directory" to 
;                                     cases where date_end > max(ti1)
;       08-Dec-2006: Viggo Hansteen - Added alternate method for computing pointing
;                                     as given by Toshifumi Shimizu (/aocs to construct_xypos)
;       10-Dec-2006: Viggo Hansteen - Added CCD temperature keywords as well as keywords for 
;                                     time arrays for each individual time file 
;                                     i.e eis_aocs1, eis_hk2, eis_sts1, (with more to come)
;       10-Feb-2007: Viggo Hansteen - Added help methods.
;       09-Mar-2007: Viggo Hansteen - Now also reads velocity keywords v_sun, v_sat, v_eth.
;        4-Jul-2008: A. Gardini     - Added HELP and INIT_EIS_SPCD_METHODS.
;       30-Apr-2009: A. Gardini     - Set keyword to eis_cal.
;       18-Feb-2010  V. Hansteen     - Implemented John Mariska's 
;                                      "EIS pointing" ver 0.95 document.
;
;  $Id: eis_spcd__define.pro 4174 2014-09-17 15:31:36Z viggoh $
;-

pro eis_spcd__define
  struct={eis_spcd, $
          help:obj_new(), $
          dir_head:'', $
          date_obs_ut:' ', $
          date_end_ut:' ', $
          date_obs:0ul, $
          date_end:0ul, $
          tr_mode:' ', $
          ufsause:' ', $
          ufsbuse:' ', $
          time_aocs1:ptr_new(), $
          time_hk2:ptr_new(), $
          time_sot:ptr_new(), $
          time_sts1:ptr_new(), $
          time_sts3:ptr_new(), $
          time_pt:ptr_new(), $
          xcen:ptr_new(), $
          ycen:ptr_new(), $
          xoffut: 0.0, $
          yoffut: 0.0, $
          xoffsu: 0.0, $
          yoffsu: 0.0, $
          sat_rot:ptr_new(), $
          inst_rot:0.0, $
          Ufsaxang:ptr_new(), $
          ufsbxang:ptr_new(), $
          ufsayang:ptr_new(), $
          ufsbyang:ptr_new(), $
          errangz:ptr_new(), $
          att_x:ptr_new(), $
          att_y:ptr_new(), $
          att_z:ptr_new(), $
          ccda_temp:ptr_new(), $
          ccdb_temp:ptr_new(), $
          mhc_ref_ther_0:ptr_new(), $
          mhc_cal_ther_0:ptr_new(), $
          mhc_hz_t10:ptr_new(), $
          mhc_hz_t15:ptr_new(), $
          v_sat:ptr_new(), $
          v_sun:ptr_new(), $
          v_eth:ptr_new(), $
          v_rst:ptr_new(), $
          crval1:ptr_new(), $
          crval2:ptr_new(), $
          crota1:ptr_new(), $
          crota2:ptr_new(), $
          ioerror:0}
end
